@@ -31,7 +31,7 @@ c**********************************************************************
      &                   a,ewt,rsp,ian,jan,igp,jgp,maxg,r,c,ic,isp,            &
      &                   maxiter,TolChange,atol,rtol,itol,Positivity,          &
      &                   Pos,ipos,SteadyStateReached,Precis,niter,             &
-     &                   dims, out,nout ,Type)
+     &                   dims, out,nout ,Type, pres)
 
 c------------------------------------------------------------------------------*
 c Solves a system of nonlinear equations using the Newton-Raphson method       *
@@ -64,7 +64,7 @@ c positivity either enforced at once (positivity=TRUE) or as a vector of element
       INTEGER  Ipos, Pos(Ipos)
 
 c tolerances, precision
-      INTEGER          itol, Type
+      INTEGER          itol, Type, pres(*)
       DOUBLE PRECISION rtol(*), atol(*),tolChange
       DOUBLE PRECISION ewt(*), precis(maxIter),maxewt,RelativeChange
          
@@ -79,18 +79,17 @@ c model and jacobian function
       DOUBLE PRECISION time
 c
       INTEGER i, j, k, esp, im, jm
-      character (len=80) msg
 c-------------------------------------------------------------------------------
       SteadyStateReached = .FALSE.
 
       CALL errSET (N, ITOL, RTOL, ATOL, SVAR, EWT)
 
 c determine sparse structure: if Type == 2, 3, 4 :
-c a 1-D or 2-D or 3-D PDE model;
+c a 1-D or 2-D or 3-D PDE model; if type = 30: 2-D model, with mapping
 c in this case the number of components, dimensions and cyclic bnd are in dims
       CALL xSparseStruct(N, nnz, ian, jan, igp, jgp, maxg, ngp,                &
      &    Svar, ewt, dSvar, beta, xmodel, time, out, nout, nonzero,            &
-     &    Type, dims)
+     &    Type, dims, pres)
 c find a minimum degree ordering of the rows and columns  
       CALL odrv(N,ian,jan,a,r,ic,nsp,isp,1,flag)
       IF (flag .NE. 0) CALL warnflag(flag,N)
@@ -195,43 +194,41 @@ c**********************************************************************
 	 SUBROUTINE warnflag(flag,N)
 	 INTEGER flag, iflag,N
 	 
- 	 character *80 msg
-
 	 iflag = INT(flag/N)
 	 IF (iflag .EQ. 1) THEN
-         write(msg,'(A10,I10)') "  row nr: ", flag-iflag
-	   call rwarn("sparse solver: null row in a")
-         call rexit(msg)
+	       call intpr ("sparse solver: null row in a", -1, 0, 0)
+         call intpr ("  row nr: ", 10, flag-iflag, 1)
+         call rexit("stopped")
        ELSE if (iflag .EQ. 2) THEN
-         write(msg,'(A10,I10)') "  row nr: ", flag-iflag
-	   call rwarn("sparse solver: duplicate entry in a")
-         call rexit(msg)
+    	   call intpr("sparse solver: duplicate entry in a", -1, 0, 0)
+         call intpr ("  row nr: ", 10, flag-iflag, 1)
+         call rexit("stopped")
        ELSE if (iflag .EQ. 3) THEN
-         write(msg,'(A10,I10)') "  row nr: ", flag-iflag
-	   call rwarn("insufficient storage in nsfc")
-         call rexit(msg)
+	       call intpr ("insufficient storage in nsfc", -1, 0, 0)
+         call intpr ("  row nr: ", 10, flag-iflag, 1)
+         call rexit("stopped")
        ELSE if (iflag .EQ. 4) THEN
-	   call rwarn("insufficient storage in nnfc")
+	       call rwarn("insufficient storage in nnfc")
        ELSE if (iflag .EQ. 5) THEN
-         write(msg,'(A10,I10)') "  row nr: ", flag-iflag
-	   call rwarn("sparse solver: null pivot")
-         call rexit(msg)
+         call rwarn("sparse solver: null pivot")
+         call intpr ("  row nr: ", 10, flag-iflag, 1)
+         call rexit("stopped")
        ELSE if (iflag .EQ. 6) THEN
-         write(msg,'(A10,I10)') "  row nr: ", flag-iflag
-  	   call rwarn("insufficient storage in nsfc")
-         call rexit(msg)
+         call intpr ("insufficient storage in nsfc", -1, 0, 0)
+         call intpr ("  row nr: ", 10, flag-iflag, 1)
+         call rexit("stopped")
        ELSE if (iflag .EQ. 7) THEN
-	   call rwarn("insufficient storage in nnfc")
+	       call rwarn("insufficient storage in nnfc")
        ELSE if (iflag .EQ. 8) THEN
-         write(msg,'(A10,I10)') "  row nr: ", flag-iflag
-         call rwarn("sparse solver: zero pivot")
-         call rexit(msg)
+         call intpr ("sparse solver: zero pivot", -1, 0, 0)
+         call intpr ("  row nr: ", 10, flag-iflag, 1)
+         call rexit("stopped")
        ELSE if (iflag .EQ. 9) THEN
-	   call rexit("insufficient storage in md")
+	       call rexit("insufficient storage in md")
        ELSE if (iflag .EQ. 10) THEN
-	   call rexit("insufficient storage in cdrv/odrv")
+	       call rexit("insufficient storage in cdrv/odrv")
        ELSE if (iflag .EQ. 11) THEN
-	   call rexit("illegal path specifications")
+	       call rexit("illegal path specifications")
        ENDIF
        RETURN
 	     END SUBROUTINE warnflag
@@ -278,7 +275,7 @@ c********************************************************************
 
       SUBROUTINE xSparseStruct(N, nnz, ian, jan, igp, jgp, maxg, ngp,          &
      &       Svar, ewt, dSvar, beta, xmodel, time, out, nout, nonzero,         &
-     &       Type, dims)
+     &       Type, dims, pres)
 c-------------------------------------------------------------------*
 c two arrays describe the sparsity structure of the jacobian:       *
 c                                                                   *
@@ -302,7 +299,7 @@ c-------------------------------------------------------------------*
 
        INTEGER           N, nnz,nonzero   
        INTEGER           IAN (N+1), JAN(nnz)
-       INTEGER           nout(*), Type, dims(*)
+       INTEGER           nout(*), Type, dims(*), pres(*)
 
        DOUBLE PRECISION  Svar (N), ewt(N)
        DOUBLE PRECISION  time, out(*), tiny
@@ -315,7 +312,6 @@ c-------------------------------------------------------------------*
        INTEGER           igp(*),jgp(N),NGP,incl(N),jdone(N)
        INTEGER           maxg,ier
        DOUBLE PRECISION  perturb
-       CHARACTER (LEN=80) msg
      
 c--------------------------------------------------------------------
        
@@ -369,8 +365,8 @@ c check memory allocation: enough?
          ENDDO
 
          IF (.not. enough) THEN
-           write (msg,'(A30,I10)')"nnz should be at least",ij
-           call rexit(msg)
+           call intpr("nnz should be at least", -1, ij, 1)
+           call rexit("stopped")
          ENDIF
          nonzero = ij
 c 1-D problem       
@@ -388,6 +384,14 @@ c 1-D problem
          cyclic(2) = dims(5)
          CALL sparse2d(N, Nspec, dimens, cyclic, nnz, ian, jan)
          nonzero = nnz
+       ELSE IF (Type == 30) THEN
+         Nspec = dims(1)
+         dimens(1) = dims(2)
+         dimens(2) = dims(3)
+         cyclic(1) = dims(4)
+         cyclic(2) = dims(5)
+         CALL sparse2dmap(N, Nspec, dimens, cyclic, nnz, ian, jan, pres)
+         nonzero = nnz
        ELSE IF (Type == 4) THEN
          Nspec = dims(1)
          dimens(1) = dims(2)
@@ -398,6 +402,16 @@ c 1-D problem
          cyclic(3) = dims(7)
          CALL sparse3d(N, Nspec, dimens, cyclic, nnz, ian, jan)
          nonzero = nnz
+       ELSE IF (Type == 40) THEN
+         Nspec = dims(1)
+         dimens(1) = dims(2)
+         dimens(2) = dims(3)
+         dimens(3) = dims(4)
+         cyclic(1) = dims(5)
+         cyclic(2) = dims(6)
+         cyclic(3) = dims(7)
+         CALL sparse3dmap(N, Nspec, dimens, cyclic, nnz, ian, jan, pres)
+         nonzero = nnz
        ENDIF
        
 c this only if jacobian estimated by calls to F
@@ -491,7 +505,6 @@ c********************************************************************
       implicit none
       INTEGER N, IA, JA, MAXG, NGRP, IGP, JGP, INCL, JDONE, IER
       DIMENSION IA(*), JA(*), IGP(*), JGP(*), INCL(*), JDONE(*)
-      CHARACTER (LEN=80) msg
 
 C-----------------------------------------------------------------------
 C This subroutine constructs groupings of the column indices of
@@ -556,10 +569,10 @@ C Error return if not all columns were chosen (MAXG too small).---------
       NG = MAXG
  70   NGRP = NG - 1
       IF (Toomuch) THEN
-        CALL rwarn("error during grouping: NGP too small")
-        WRITE (msg,'(A30,I10,A10,I10)')"Should be at least",NGRP,              &
-     &   "is",maxG
-        CALL rexit(msg)
+        CALL intpr("error during grouping: NGP too small",-1,0,0)
+        CALL intpr("Should be at least: ",-1, NGRP,1)
+        CALL intpr("while it is ", -1, maxG, 1)
+        CALL rexit("stopped")
       ENDIF
       RETURN
  80   IER = 1
